@@ -19,16 +19,20 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
-import pipes
 import re
 import os
+import pipes
 import tempfile
 
 DOCUMENTATION = """
 ---
 module: lineinfile
-author: Daniel Hokka Zakrisson, Ahti Kitsik
-extends_documentation_fragment: files
+author:
+    - "Daniel Hokka Zakrissoni (@dhozac)"
+    - "Ahti Kitsik (@ahtik)"
+extends_documentation_fragment:
+    - files
+    - validate
 short_description: Ensure a particular line is in a file, or replace an
                    existing line using a back-referenced regular expression.
 description:
@@ -65,8 +69,7 @@ options:
     description:
       - Required for C(state=present). The line to insert/replace into the
         file. If C(backrefs) is set, may contain backreferences that will get
-        expanded with the C(regexp) capture groups if the regexp matches. The
-        backreferences should be double escaped (see examples).
+        expanded with the C(regexp) capture groups if the regexp matches.
   backrefs:
     required: false
     default: "no"
@@ -115,16 +118,6 @@ options:
      description:
        - Create a backup file including the timestamp information so you can
          get the original file back if you somehow clobbered it incorrectly.
-  validate:
-     required: false
-     description:
-       - validation to run before copying into place. 
-         Use %s in the command to indicate the current file to validate.
-         The command is passed securely so shell features like
-         expansion and pipes won't work.
-     required: false
-     default: None
-     version_added: "1.4"
   others:
      description:
        - All arguments accepted by the M(file) module also work here.
@@ -244,8 +237,11 @@ def present(module, dest, regexp, line, insertafter, insertbefore, create,
             # Don't do backref expansion if not asked.
             new_line = line
 
-        if lines[index[0]] != new_line + os.linesep:
-            lines[index[0]] = new_line + os.linesep
+        if not new_line.endswith(os.linesep):
+            new_line += os.linesep
+
+        if lines[index[0]] != new_line:
+            lines[index[0]] = new_line
             msg = 'line replaced'
             changed = True
     elif backrefs:
@@ -370,26 +366,6 @@ def main():
 
         line = params['line']
 
-        # The safe_eval call will remove some quoting, but not others,
-        # so we need to know if we should specifically unquote it.
-        should_unquote = not is_quoted(line)
-
-        # always add one layer of quotes
-        line = "'%s'" % line
-
-        # Replace escape sequences like '\n' while being sure 
-        # not to replace octal escape sequences (\ooo) since they
-        # match the backref syntax.
-        if backrefs:
-            line = re.sub(r'(\\[0-9]{1,3})', r'\\\1', line)
-        line = module.safe_eval(line)
-
-        # Now remove quotes around the string, if needed after
-        # removing the layer we added above
-        line = unquote(line)
-        if should_unquote:
-            line = unquote(line)
-
         present(module, dest, params['regexp'], line,
                 ins_aft, ins_bef, create, backup, backrefs)
     else:
@@ -401,5 +377,5 @@ def main():
 # import module snippets
 from ansible.module_utils.basic import *
 from ansible.module_utils.splitter import *
-
-main()
+if __name__ == '__main__':
+    main()
