@@ -20,10 +20,13 @@
 #
 # see examples/playbooks/uri.yml
 
+import cgi
 import shutil
 import tempfile
 import base64
 import datetime
+from distutils.version import LooseVersion
+
 try:
     import json
 except ImportError:
@@ -43,7 +46,6 @@ options:
       - HTTP or HTTPS URL in the form (http|https)://host.domain[:port]/path
     required: true
     default: null
-    aliases: []
   dest:
     description:
       - path of where to download the file to (if desired). If I(dest) is a directory, the basename of the file on the remote server will be used.
@@ -64,11 +66,18 @@ options:
       - The body of the http request/response to the web service.
     required: false
     default: null
+  body_format:
+    description:
+      - The serialization format of the body. When set to json, encodes the body argument and automatically sets the Content-Type header accordingly.
+    required: false
+    choices: [ "raw", "json" ]
+    default: raw
+    version_added: "2.0"
   method:
     description:
-      - The HTTP method of the request or response.
+      - The HTTP method of the request or response. It MUST be uppercase.
     required: false
-    choices: [ "GET", "POST", "PUT", "HEAD", "DELETE", "OPTIONS", "PATCH" ]
+    choices: [ "GET", "POST", "PUT", "HEAD", "DELETE", "OPTIONS", "PATCH", "TRACE", "CONNECT", "REFRESH" ]
     default: "GET"
   return_content:
     description:
@@ -136,8 +145,9 @@ options:
     version_added: '1.9.2'
 
 # informational: requirements for nodes
-requirements: [ urlparse, httplib2 ]
-author: Romeo Theriault
+requirements:
+  - httplib2 >= 0.7.0
+author: "Romeo Theriault (@romeotheriault)"
 '''
 
 EXAMPLES = '''
@@ -149,39 +159,57 @@ EXAMPLES = '''
   register: webpage
 
 - action: fail
-  when: 'AWESOME' not in "{{ webpage.content }}"
+  when: "'AWESOME' not in webpage.content"
 
 
 # Create a JIRA issue
-
-- uri: url=https://your.jira.example.com/rest/api/2/issue/ 
-       method=POST user=your_username password=your_pass 
-       body="{{ lookup('file','issue.json') }}" force_basic_auth=yes 
-       status_code=201 HEADER_Content-Type="application/json"  
+- uri:
+    url: https://your.jira.example.com/rest/api/2/issue/ 
+    method: POST
+    user: your_username 
+    password: your_pass 
+    body: "{{ lookup('file','issue.json') }}"
+    force_basic_auth: yes 
+    status_code: 201
+    body_format: json 
 
 # Login to a form based webpage, then use the returned cookie to
 # access the app in later tasks
 
-- uri: url=https://your.form.based.auth.examle.com/index.php 
-       method=POST body="name=your_username&password=your_password&enter=Sign%20in" 
-       status_code=302 HEADER_Content-Type="application/x-www-form-urlencoded"
+- uri:
+    url: https://your.form.based.auth.examle.com/index.php 
+    method: POST
+    body: "name=your_username&password=your_password&enter=Sign%20in" 
+    status_code: 302
+    HEADER_Content-Type: "application/x-www-form-urlencoded"
   register: login
 
-- uri: url=https://your.form.based.auth.example.com/dashboard.php
-       method=GET return_content=yes HEADER_Cookie="{{login.set_cookie}}"
+- uri:
+    url: https://your.form.based.auth.example.com/dashboard.php
+    method: GET
+    return_content: yes
+    HEADER_Cookie: "{{login.set_cookie}}"
 
 # Queue build of a project in Jenkins:
-
-- uri: url=http://{{jenkins.host}}/job/{{jenkins.job}}/build?token={{jenkins.token}} 
-       method=GET user={{jenkins.user}} password={{jenkins.password}} force_basic_auth=yes status_code=201
+- uri:
+    url: "http://{{ jenkins.host }}/job/{{ jenkins.job }}/build?token={{ jenkins.token }}" 
+    method: GET
+    user: "{{ jenkins.user }}"
+    password: "{{ jenkins.password }}"
+    force_basic_auth: yes
+    status_code: 201
 
 '''
 
-HAS_HTTPLIB2 = True
+HAS_HTTPLIB2 = False
+
 try:
     import httplib2
-except ImportError:
-    HAS_HTTPLIB2 = False
+    if LooseVersion(httplib2.__version__) >= LooseVersion('0.7'):
+        HAS_HTTPLIB2 = True
+except ImportError, AttributeError:
+    # AttributeError if __version__ is not present
+    pass
 
 HAS_URLPARSE = True
 
@@ -191,7 +219,6 @@ try:
 except ImportError:
     HAS_URLPARSE = False
 
-
 def write_file(module, url, dest, content):
     # create a tempfile with some test content
     fd, tmpsrc = tempfile.mkstemp()
@@ -247,11 +274,11 @@ def url_filename(url):
     return fn
 
 
-def uri(module, url, dest, user, password, body, method, headers, redirects, socket_timeout, validate_certs):
+def uri(module, url, dest, user, password, body, body_format, method, headers, redirects, socket_timeout, validate_certs):
     # To debug
-    #httplib2.debug = 4
+    #httplib2.debuglevel = 4
 
-    # Handle Redirects         
+    # Handle Redirects
     if redirects == "all" or redirects == "yes":
         follow_redirects = True
         follow_all_redirects = True
@@ -313,10 +340,7 @@ def uri(module, url, dest, user, password, body, method, headers, redirects, soc
         r['redirected'] = redirected
         r.update(resp_redir)
         r.update(resp)
-        try:
-            return r, unicode(content.decode('raw_unicode_escape')), dest
-        except:
-            return r, content, dest
+        return r, content, dest
     except httplib2.RedirectMissingLocation:
         module.fail_json(msg="A 3xx redirect response code was provided but no Location: header was provided to point to the new location.")
     except httplib2.RedirectLimit:
@@ -349,7 +373,8 @@ def main():
             user = dict(required=False, default=None),
             password = dict(required=False, default=None),
             body = dict(required=False, default=None),
-            method = dict(required=False, default='GET', choices=['GET', 'POST', 'PUT', 'HEAD', 'DELETE', 'OPTIONS', 'PATCH']),
+            body_format = dict(required=False, default='raw', choices=['raw', 'json']),
+            method = dict(required=False, default='GET', choices=['GET', 'POST', 'PUT', 'HEAD', 'DELETE', 'OPTIONS', 'PATCH', 'TRACE', 'CONNECT', 'REFRESH']),
             return_content = dict(required=False, default='no', type='bool'),
             force_basic_auth = dict(required=False, default='no', type='bool'),
             follow_redirects = dict(required=False, default='safe', choices=['all', 'safe', 'none', 'yes', 'no']),
@@ -364,7 +389,7 @@ def main():
     )
 
     if not HAS_HTTPLIB2:
-        module.fail_json(msg="httplib2 is not installed")
+        module.fail_json(msg="httplib2 >= 0.7 is not installed")
     if not HAS_URLPARSE:
         module.fail_json(msg="urlparse is not installed")
 
@@ -372,6 +397,7 @@ def main():
     user = module.params['user']
     password = module.params['password']
     body = module.params['body']
+    body_format = module.params['body_format']
     method = module.params['method']
     dest = module.params['dest']
     return_content = module.params['return_content']
@@ -383,14 +409,21 @@ def main():
     socket_timeout = module.params['timeout']
     validate_certs = module.params['validate_certs']
 
-    # Grab all the http headers. Need this hack since passing multi-values is currently a bit ugly. (e.g. headers='{"Content-Type":"application/json"}')
     dict_headers = {}
+
+    # If body_format is json, encodes the body (wich can be a dict or a list) and automatically sets the Content-Type header
+    if body_format == 'json':
+        body = json.dumps(body)
+        dict_headers['Content-Type'] = 'application/json'
+
+
+    # Grab all the http headers. Need this hack since passing multi-values is currently a bit ugly. (e.g. headers='{"Content-Type":"application/json"}')
     for key, value in module.params.iteritems():
         if key.startswith("HEADER_"):
             skey = key.replace("HEADER_", "")
             dict_headers[skey] = value
 
-  
+
     if creates is not None:
         # do not run the command if the line contains creates=filename
         # and the filename already exists.  This allows idempotence
@@ -416,7 +449,7 @@ def main():
 
 
     # Make the request
-    resp, content, dest = uri(module, url, dest, user, password, body, method, dict_headers, redirects, socket_timeout, validate_certs)
+    resp, content, dest = uri(module, url, dest, user, password, body, body_format, method, dict_headers, redirects, socket_timeout, validate_certs)
     resp['status'] = int(resp['status'])
 
     # Write the file out if requested
@@ -438,25 +471,35 @@ def main():
     # Transmogrify the headers, replacing '-' with '_', since variables dont work with dashes.
     uresp = {}
     for key, value in resp.iteritems():
-        ukey = key.replace("-", "_")  
+        ukey = key.replace("-", "_")
         uresp[ukey] = value
 
+    # Default content_encoding to try
+    content_encoding = 'utf-8'
     if 'content_type' in uresp:
-        if uresp['content_type'].startswith('application/json') or \
-                uresp['content_type'].startswith('text/json'):
+        content_type, params = cgi.parse_header(uresp['content_type'])
+        if 'charset' in params:
+            content_encoding = params['charset']
+        u_content = unicode(content, content_encoding, errors='replace')
+        if content_type.startswith('application/json') or \
+                content_type.startswith('text/json'):
             try:
-                js = json.loads(content)
+                js = json.loads(u_content)
                 uresp['json'] = js
             except:
                 pass
+    else:
+        u_content = unicode(content, content_encoding, errors='replace')
+
     if resp['status'] not in status_code:
-        module.fail_json(msg="Status code was not " + str(status_code), content=content, **uresp)
+        module.fail_json(msg="Status code was not " + str(status_code), content=u_content, **uresp)
     elif return_content:
-        module.exit_json(changed=changed, content=content, **uresp)
+        module.exit_json(changed=changed, content=u_content, **uresp)
     else:
         module.exit_json(changed=changed, **uresp)
 
 
 # import module snippets
 from ansible.module_utils.basic import *
-main()
+if __name__ == '__main__':
+    main()
