@@ -24,25 +24,21 @@ options:
   mode:
     required: false
     default: null
-    choices: []
     description:
-      - mode the file or directory should be, such as 0644 as would be fed to I(chmod)
+      - mode the file or directory should be. For those used to I(/usr/bin/chmod) remember that modes are actually octal numbers (like 0644). Leaving off the leading zero will likely have unexpected results. As of version 1.8, the mode may be specified as a symbolic mode (for example, C(u+rwx) or C(u=rw,g=r,o=r)).
   owner:
     required: false
     default: null
-    choices: []
     description:
       - name of the user that should own the file/directory, as would be fed to I(chown)
   group:
     required: false
     default: null
-    choices: []
     description:
       - name of the group that should own the file/directory, as would be fed to I(chown)
   seuser:
     required: false
     default: null
-    choices: []
     description:
       - user part of SELinux file context. Will default to system policy, if
         applicable. If set to C(_default), it will use the C(user) portion of the
@@ -50,21 +46,26 @@ options:
   serole:
     required: false
     default: null
-    choices: []
     description:
       - role part of SELinux file context, C(_default) feature works as for I(seuser).
   setype:
     required: false
     default: null
-    choices: []
     description:
       - type part of SELinux file context, C(_default) feature works as for I(seuser).
   selevel:
     required: false
     default: "s0"
-    choices: []
     description:
       - level part of the SELinux file context. This is the MLS/MCS attribute,
         sometimes known as the C(range). C(_default) feature works as for
         I(seuser).
+  follow:
+    required: false
+    default: "no"
+    choices: [ "yes", "no" ]
+    version_added: "1.8"
+    description:
+      - 'This flag indicates that filesystem links, if they exist, should be followed.'
+
 """
